/*
** -- Local Includes --
*/

#include "qtchess_gui.h"

/*
** -- Qt Includes --
*/

#include <QMenu>
#include <QFrame>
#include <QLabel>
#include <QCloseEvent>
#include <QGridLayout>
#include <QHBoxLayout>

extern qtchess *chess;
extern qtchess_gui *gui;
extern qtchess_comm *comm;
extern QApplication *qapp;

int qtchess_gui::exec(void)
{
  return qapp->exec();
}

void qtchess_gui::setStatusText(const QString &str)
{
  statusLabel->setText(str);
}

void qtchess_gui::init(void)
{
  QFrame *frame = NULL;
  QMenu *filemenu = NULL, *helpmenu = NULL, *viewmenu = NULL,
    *setupmenu = NULL;

  setWindowTitle("QtChess");

  /*
  ** Add a menubar.
  */

  if((filemenu = new QMenu("&File")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((setupmenu = new QMenu("&Setup")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((viewmenu = new QMenu("&View")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((statusLabel = new QLabel("Status: Ready")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  statusLabel->setMargin(0);
  statusLabel->setFrameStyle(QFrame::NoFrame);
  statusBar()->setSizeGripEnabled(FALSE);
  statusBar()->addWidget(statusLabel, 100);
  connect(filemenu->addAction("&New Game"),
	  SIGNAL(triggered()), this, SLOT(newGame()));
  filemenu->addSeparator();
  connect(filemenu->addAction("&Quit"),
	  SIGNAL(triggered()), this, SLOT(quit()));
  connect(setupmenu->addAction("&Connection Configuration"),
	  SIGNAL(triggered()), this, SLOT(setup()));
  connect(viewmenu->addAction("&Clocks"),
	  SIGNAL(triggered()), this, SLOT(showClocks()));
  connect(viewmenu->addAction("&History"),
	  SIGNAL(triggered()), this, SLOT(history()));

  if((helpmenu = new QMenu("&Help")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  connect(helpmenu->addAction("&About"),
	  SIGNAL(triggered()), this, SLOT(about()));
  connect(helpmenu->addAction("&Help"),
	  SIGNAL(triggered()), this, SLOT(help()));

  if((help_dialog = new qtchess_help_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((setup_dialog = new qtchess_setup_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((history_dialog = new qtchess_history_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((promote_dialog = new qtchess_promote_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((clocks_dialog = new qtchess_clocks_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  menuBar()->addMenu(filemenu);
  menuBar()->addMenu(setupmenu);
  menuBar()->addMenu(viewmenu);
  menuBar()->addSeparator();
  menuBar()->addMenu(helpmenu);

  if((frame = new QFrame(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((glboard = new openglWid(frame)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  frame->setFrameStyle(QFrame::NoFrame);
  frame->setLineWidth(2);

  /*
  ** Add the OpenGL board.
  */

  glboard->resize(OPEN_GL_DIMENSION, OPEN_GL_DIMENSION);
  frame->setFixedSize(QSize(OPEN_GL_DIMENSION, OPEN_GL_DIMENSION));
  setCentralWidget(frame);
  layout()->setSizeConstraint(QLayout::SetFixedSize);
  show();
}

void qtchess_gui::display(void)
{
}

void qtchess_gui::showClocks(void)
{
  clocks_dialog->hide();
  clocks_dialog->focus();
  clocks_dialog->show();
}

void qtchess_gui::history(void)
{
  history_dialog->hide();
  history_dialog->focus();
  history_dialog->show();
}

void qtchess_gui::about(void)
{
  (void) QMessageBox::information
    (this,
     "About",
     "QtChess Version 3.03.\n"
     "Copyright (c) 2003, 2004, 2006, 2007 "
     "Alexis Megas.",
     QMessageBox::Ok | QMessageBox::Default);
}

void qtchess_gui::showErrorMsg(const char *message)
{
  (void) QMessageBox::critical(this, "Error", message,
			       QMessageBox::Ok,
			       QMessageBox::Default);
}

void qtchess_gui::help(void)
{
  help_dialog->setup();
}

void qtchess_gui::quit(void)
{
  chess->quit(NULL, EXIT_SUCCESS);
}

void qtchess_gui::startTimers(const int which)
{
  clocks_dialog->startTimers(which);
}

void qtchess_gui::stopTimers(const int which)
{
  clocks_dialog->stopTimers(which);
}

void qtchess_gui::closeEvent(QCloseEvent *e)
{
  (void) e;
  chess->quit(NULL, EXIT_SUCCESS);
}

void qtchess_gui::newGame(void)
{
  int i = 0, j = 0;
  struct move_s current_move;

  if(chess->getFirst() == -1)
    {
      showErrorMsg("You must first start a game.");
      return;
    }
  else if(chess->getFirst() != I_AM_FIRST)
    {
      showErrorMsg("Only the first player may initiate a new game.");
      return;
    }

  chess->init();
  gui->getGLBoard()->updateGL();      

  for(i = 0; i < NSQUARES; i++)
    for(j = 0; j < NSQUARES; j++)
      current_move.board[i][j] = chess->board[i][j];

  current_move.x1 = current_move.x2 = current_move.y1 =
    current_move.y2 = current_move.r_x1 =
    current_move.r_x2 = current_move.r_y1 =
    current_move.r_y2 = current_move.piece =
    current_move.rook = -1;
  current_move.promoted = current_move.pawn_2 = 0;
  comm->sendMove(current_move);
  history_dialog->clear_method();
  clocks_dialog->init();

  /*
  ** Careful here. Set this after calling sendMove().
  */

  chess->setTurn(MY_TURN);
}

void qtchess_gui::setup(void)
{
  setup_dialog->getRHostField()->setFocus();
  setup_dialog->exec();
}

qtchess_setup_dialog *qtchess_gui::getSetupDialog(void)
{
  return setup_dialog;
}

qtchess_history_dialog *qtchess_gui::getHistoryDialog(void)
{
  return history_dialog;
}

qtchess_promote_dialog *qtchess_gui::getPromoteDialog(void)
{
  return promote_dialog;
}

void qtchess_gui::notifyConnection(const QString &address)
{
  statusLabel->setText("Status: Host " + address + " Connected");
}

void qtchess_gui::showGameOver(const int turn)
{
  (void) turn;
  (void) QMessageBox::information
    (this,
     "Game Over",
     "Game Over. Please enjoy another game.",
     QMessageBox::Ok | QMessageBox::Default);
}

void qtchess_gui::showDisconnect(void)
{
  setup_dialog->disconnectedState();
  statusLabel->setText("Status: Ready");
  clocks_dialog->stopTimers(PLAYER_TIMER);
  clocks_dialog->stopTimers(OPPONENT_TIMER);
}

void qtchess_gui::showNewGameInfo(void)
{
}

void qtchess_gui::initClocks(void)
{
  clocks_dialog->init();
}

void qtchess_setup_dialog::ok_cb(void)
{
  QString str1 = "", str2 = "";

  /*
  ** Make sure the user is not connecting to this host.
  */

  str1 = ui.rhost->text();
  str1 = str1.trimmed();
  str2 = ui.rport->text();
  str2 = str2.trimmed();

  if(str1.length() == 0 || str2.length() == 0)
    return;

  comm->connectRemotely();

  if(comm->isSet())
    {
      ui.ok->setEnabled(FALSE);
      ui.rhost->setReadOnly(TRUE);
      ui.rport->setReadOnly(TRUE);
      ui.rhost->setEnabled(TRUE);
      ui.rport->setEnabled(TRUE);
      ui.disconnect->setEnabled(TRUE);
    }
  else
    {
      ui.ok->setEnabled(TRUE);
      ui.rhost->setReadOnly(FALSE);
      ui.rport->setReadOnly(FALSE);
      ui.rhost->setEnabled(TRUE);
      ui.rport->setEnabled(TRUE);
      ui.disconnect->setEnabled(FALSE);
    }
}

void qtchess_setup_dialog::close_cb(void)
{
  hide();
}

void qtchess_setup_dialog::disconnect_cb(void)
{
  if(comm->isSet())
    {
      comm->disconnectRemotely();

      if(!comm->isSet())
	{
	  gui->setStatusText("Status: Ready");
	  disconnectedState();
	}
    }
}

void qtchess_setup_dialog::disconnectedState(void)
{
  ui.ok->setEnabled(TRUE);
  ui.disconnect->setEnabled(FALSE);
  ui.rhost->setReadOnly(FALSE);
  ui.rport->setReadOnly(FALSE);
  ui.rhost->setEnabled(TRUE);
  ui.rport->setEnabled(TRUE);
}

qtchess_setup_dialog::qtchess_setup_dialog(QWidget *parent):
  QDialog(parent)
{
  ui.setupUi(this);
  ui.disconnect->setEnabled(FALSE);
  connect(ui.cancel, SIGNAL(clicked()), this, SLOT(close_cb()));
  connect(ui.ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
  connect(ui.disconnect, SIGNAL(clicked()), this, SLOT(disconnect_cb()));
}

QLineEdit *qtchess_setup_dialog::getHostField(void)
{
  return ui.host;
}

QLineEdit *qtchess_setup_dialog::getPortField(void)
{
  return ui.port;
}

QLineEdit *qtchess_setup_dialog::getRHostField(void)
{
  return ui.rhost;
}

QLineEdit *qtchess_setup_dialog::getRPortField(void)
{
  return ui.rport;
}

void qtchess_history_dialog::ok_cb(void)
{
  hide();
}

void qtchess_history_dialog::clear_cb(void)
{
  clear_method();
}

void qtchess_history_dialog::clear_method(void)
{
  ui.text->clear();
}

void qtchess_history_dialog::add_move(const struct move_s current_move,
				      const int color)
{
  char msg[512];
  char to_let[2];
  char from_let[2];
  QString txt = "";
  QDateTime now = QDateTime::currentDateTime();

  (void) memset(msg, 0, sizeof(msg));
  (void) memset(to_let, 0, sizeof(to_let));
  (void) memset(from_let, 0, sizeof(from_let));
  from_let[0] = (char) (65 + current_move.y1);
  to_let[0] = (char ) (65 + current_move.y2);

#ifdef _DEBUG_
  (void) fprintf(stderr, "from_let = %s, to_let = %s\n", from_let, to_let);
#endif

  if(qtchess_validate::isKing(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d%s", "King",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1,
		    current_move.r_x1 != -1 ? " (castling)" : "");
  else if(qtchess_validate::isPawn(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Pawn",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else if(qtchess_validate::isRook(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Rook",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else if(qtchess_validate::isBishop(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Bishop",
		   from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else if(qtchess_validate::isQueen(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Queen",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Knight",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);

  if(color == BLACK)
    txt = QString("%1: Black %2").arg
      (now.toString("MM/dd/yyyy hh:mm:ss")).arg(msg);
  else
    txt = QString("%1: White %2").arg
      (now.toString("MM/dd/yyyy hh:mm:ss")).arg(msg);

  if(current_move.promoted)
    txt.append(" (promotion)");

  txt.append(".");
  ui.text->append(txt);
}

qtchess_history_dialog::qtchess_history_dialog(QWidget *parent):
  QDialog(parent)
{
  ui.setupUi(this);
  connect(ui.ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
  connect(ui.clear, SIGNAL(clicked()), this, SLOT(clear_cb()));
}

void qtchess_promote_dialog::ok_cb(void)
{
  hide();
}

void qtchess_promote_dialog::setup(void)
{
  ui.menu->setFocus();
  exec();
}

qtchess_promote_dialog::qtchess_promote_dialog(QWidget *parent):
  QDialog(parent)
{
  ui.setupUi(this);
  connect(ui.ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
}

void qtchess_help_dialog::ok_cb(void)
{
  hide();
}

void qtchess_help_dialog::setup(void)
{
  ui.ok->setFocus();
  exec();
}

qtchess_help_dialog::qtchess_help_dialog(QWidget *parent):
  QDialog(parent)
{
  ui.setupUi(this);
  connect(ui.ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
  setMinimumWidth(600);
  setMinimumHeight(300);
  ui.text->append("General Guidelines\n"
		  "- Please do not connect to the same process.\n"
		  "- Beige always moves first.\n"
		  "- King is allowed to move to a threatened square, "
		  "unless castling.\n"
		  "- Moves are prohibited until connections have been "
		  "established.");
}

void qtchess_clocks_dialog::ok_cb(void)
{
  hide();
}

qtchess_clocks_dialog::qtchess_clocks_dialog(QWidget *parent):
  QDialog(parent)
{
  ui.setupUi(this);
  connect(ui.ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
  playert = NULL;
  opponentt = NULL;
  ui.player->setDisplayFormat("hh:mm:ss");
  ui.opponent->setDisplayFormat("hh:mm:ss");

  if((playert = new QTimer()) != NULL)
    connect(playert, SIGNAL(timeout()), SLOT(updatePlayer()));

  if((opponentt = new QTimer()) != NULL)
    connect(opponentt, SIGNAL(timeout()), SLOT(updateOpponent()));

  if(playert != NULL)
    playert->start(1000);

  if(opponentt != NULL)
    opponentt->start(1000);
}

void qtchess_clocks_dialog::startTimers(const int which)
{
  (void) which;
}

void qtchess_clocks_dialog::stopTimers(const int which)
{
  (void) which;
}

void qtchess_clocks_dialog::updatePlayer(void)
{
  if(comm->isReady() && chess->getTurn() == MY_TURN && !chess->isGameOver())
    ui.player->setTime(ui.player->time().addSecs(1));
}

void qtchess_clocks_dialog::updateOpponent(void)
{
  if(comm->isReady() && chess->getTurn() == THEIR_TURN && !chess->isGameOver())
    ui.opponent->setTime(ui.opponent->time().addSecs(1));
}

void qtchess_clocks_dialog::init(void)
{
  ui.player->setTime(QTime(0, 0, 0));
  ui.opponent->setTime(QTime(0, 0, 0));
}
